package ai;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

import ai.queens.QueenAnnealingResolver;
import ai.queens.QueenGeneticResolver;

/**
 * @author Yoann Eichelberger - M1 IWOCS
 */
public class App {

    private static Map<String, Function<Integer, Resolver<int[]>>> metaHeuristics;

    static {
        metaHeuristics = new HashMap<>();
        metaHeuristics.put("annealing", QueenAnnealingResolver::new);
        metaHeuristics.put("genetic", QueenGeneticResolver::new);
    }

    /**
     * Lance le programme.
     */
    public static void main(String[] args) {
        String name = args.length > 0 ? args[0] : "genetic";
        int min = args.length > 1 ? Integer.parseInt(args[1]) : 4;
        int max = args.length > 2 ? Integer.parseInt(args[2]) : args.length == 2 ? min + 1 : 100;
        long start, end;
        for (int i = min; i < max; i++) {
            Resolver<int[]> resolver = metaHeuristics.get(name).apply(i);
            start = System.currentTimeMillis();
            int[] solution = resolver.search();
            end = System.currentTimeMillis();
            show(solution);
            System.out.printf("i: %d time: %dms%n", i, (end - start));
        }
    }

    /**
     * Vérifie si un emplacement est en prise sur un échiquier.
     * 
     * @param queens échiquier.
     * @param l      ligne de l'emplacement.
     * @param c      colonne de l'emplacement.
     * @return true si l'emplacement est en prise, sinon false.
     */
    public static boolean isTaken(int[] queens, int l, int c) {
        for (int j = 0; j < l; j++) {
            if (queens[j] == c || l - j == c - queens[j] || l - j == queens[j] - c) {
                return true;
            }
        }
        return false;
    }

    /**
     * Permet de permuter des entiers dans un tableau.
     * 
     * @param array tableau d'entiers.
     * @param a     premier indice.
     * @param b     deuxième indice.
     */
    public static void permutate(int[] array, int a, int b) {
        int tmp = array[a];
        array[a] = array[b];
        array[b] = tmp;
    }

    /**
     * Affiche la solution à l'utilisateur.
     * 
     * @param queens tableau de reines.
     */
    public static void show(int[] queens) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < queens.length; i++) {
            for (int j = 0; j < queens.length; j++) {
                sb.append(j == queens[i] ? "X" : " ");
                sb.append('|');
            }
            sb.append('\n');
        }
        sb.append('\n');
        System.out.println(sb.toString());
    }
}
