package ai.metaheuristic.annealing;

import static java.lang.Math.exp;
import static java.lang.Math.pow;
import static java.lang.Math.random;

/**
 * @author Yoann Eichelberger - M1 IWOCS
 */
public abstract class Annealing<V> {

    private static final float Tratio = .000001F;

    private float T0;
    private float Nt;
    private float Rt;

    /**
     * 
     * @param T0
     * @param Nt
     */
    public Annealing(float T0, float Nt) {
        this.T0 = T0;
        this.Nt = Nt;
        this.Rt = (float) pow(Tratio, 1F / Nt);
    }

    /**
     * 
     */
    public float decrease(float T) {
        return T * this.Rt;
    }

    /**
     * 
     */
    public V recuit(V X0) {
        V X = X0, Y;
        float T = T0;
        float dF, fY = 1;
        while (!(T < Annealing.Tratio * this.T0) && fY != 0) {
            for (int m = 0; m < this.Nt && fY != 0; m++) {
                Y = neighbour(X);
                fY = fitness(Y);
                dF = fY - fitness(X);
                if (accept(dF, T)) {
                    X = Y;
                }
            }
            T = decrease(T);
        }
        return X;
    }

    /**
     * 
     */
    public boolean accept(float dF, float T) {
        return dF < 0 || random() < exp(-dF / T);
    }

    /**
     * 
     */
    public abstract float fitness(V x);

    /**
     * 
     */
    public abstract V neighbour(V X);
}