package ai.queens;

import static java.lang.Math.floor;
import static java.lang.Math.random;

import java.util.Arrays;

import ai.App;
import ai.Resolver;
import ai.metaheuristic.annealing.Annealing;

/**
 * @author Yoann Eichelberger - M1 IWOCS
 */
public class QueenAnnealingResolver implements Resolver<int[]> {

    private int[] queens;
    private int size;
    private Annealing<int[]> annealing;

    /**
     * Construit une instance d'un resolver d'un problème à N-Reines avec le recuit
     * simulé.
     * 
     * @param size nombre de reines.
     */
    public QueenAnnealingResolver(int size) {
        this.queens = new int[size];
        this.size = size;
        for (int i = 0; i < this.size; i++) {
            this.queens[i] = i;
        }
        this.annealing = new Annealing<int[]>(100F, 200F) {
            @Override
            public float fitness(int[] queens) {
                int c = 0;
                for (int i = 1; i < queens.length; i++) {
                    if (App.isTaken(queens, i, queens[i])) {
                        c++;
                    }
                }
                return c;
            }

            @Override
            public int[] neighbour(int[] X) {
                if (X.length <= 1) {
                    return X;
                }
                int n = X.length;
                int a = (int) (floor(random() * n));
                int b = (int) (a + 1 + floor(random() * n)) % n;
                int[] copy = Arrays.copyOf(X, n);
                App.permutate(copy, a, b);
                return copy;
            }
        };
    }

    /**
     * Recherche une solution.
     * 
     * @return le tableau de reines répondant au problème.
     */
    @Override
    public int[] search() {
        int[] x = this.annealing.recuit(this.queens);
        return this.annealing.fitness(x) == 0 ? x : null;
    }
}